/*
 * Name:    d_delta.c 
 *
 * Purpose: MX motor driver to move a real motor relative to a fixed position.
 *
 *----------------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "mx_util.h"
#include "mx_drivr.h"
#include "mx_motor.h"
#include "mx_var.h"
#include "d_delta.h"

/* Initialize the motor driver jump table. */

MX_RECORD_FUNCTION_LIST mxd_delta_motor_record_function_list = {
	mxd_delta_motor_initialize_type,
	mxd_delta_motor_create_record_structures,
	mxd_delta_motor_finish_record_initialization,
	mxd_delta_motor_delete_record,
	mxd_delta_motor_print_motor_structure,
	mxd_delta_motor_read_parms_from_hardware,
	mxd_delta_motor_write_parms_to_hardware,
	mxd_delta_motor_open,
	mxd_delta_motor_close
};

MX_MOTOR_FUNCTION_LIST mxd_delta_motor_motor_function_list = {
	mxd_delta_motor_motor_is_busy,
	mxd_delta_motor_move_absolute,
	mxd_delta_motor_get_position,
	mxd_delta_motor_set_position,
	mxd_delta_motor_soft_abort,
	mxd_delta_motor_immediate_abort,
	mxd_delta_motor_positive_limit_hit,
	mxd_delta_motor_negative_limit_hit,
	mxd_delta_motor_find_home_position
};

/* Delta motor data structures. */

MX_RECORD_FIELD_DEFAULTS mxd_delta_motor_record_field_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_ANALOG_MOTOR_STANDARD_FIELDS,
	MX_MOTOR_STANDARD_FIELDS,
	MXD_DELTA_MOTOR_STANDARD_FIELDS
};

long mxd_delta_motor_num_record_fields
		= sizeof( mxd_delta_motor_record_field_defaults )
			/ sizeof( mxd_delta_motor_record_field_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxd_delta_motor_rfield_def_ptr
			= &mxd_delta_motor_record_field_defaults[0];

static mx_status_type
mxd_delta_motor_get_fixed_position_value( MX_DELTA_MOTOR *delta_motor,
					double *fixed_position_value );

/* === */

MX_EXPORT mx_status_type
mxd_delta_motor_initialize_type( long type )
{
		/* Nothing needed here. */

		return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_delta_motor_create_record_structures( MX_RECORD *record )
{
	const char fname[] = "mxd_delta_motor_create_record_structures()";

	MX_MOTOR *motor;
	MX_DELTA_MOTOR *delta_motor;

	/* Allocate memory for the necessary structures. */

	motor = (MX_MOTOR *) malloc( sizeof(MX_MOTOR) );

	if ( motor == (MX_MOTOR *) NULL ) {
		return mx_error( MXE_OUT_OF_MEMORY, fname,
		"Can't allocate memory for MX_MOTOR structure." );
	}

	delta_motor = (MX_DELTA_MOTOR *) malloc( sizeof(MX_DELTA_MOTOR) );

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_OUT_OF_MEMORY, fname,
		"Can't allocate memory for MX_DELTA_MOTOR structure." );
	}

	/* Now set up the necessary pointers. */

	record->record_class_struct = motor;
	record->record_type_struct = delta_motor;
	record->class_specific_function_list
				= &mxd_delta_motor_motor_function_list;

	motor->record = record;
	delta_motor->motor = motor;

	/* A delta motor is treated as an analog motor. */

	motor->subclass = MXC_MTR_ANALOG;

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_delta_motor_finish_record_initialization( MX_RECORD *record )
{
	mx_status_type status;

	status = mx_motor_finish_record_initialization( record );

	return status;
}

MX_EXPORT mx_status_type
mxd_delta_motor_delete_record( MX_RECORD *record )
{
	if ( record == NULL ) {
		return MX_SUCCESSFUL_RESULT;
	}
	if ( record->record_type_struct != NULL ) {
		free( record->record_type_struct );

		record->record_type_struct = NULL;
	}
	if ( record->record_class_struct != NULL ) {
		free( record->record_class_struct );

		record->record_class_struct = NULL;
	}
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_delta_motor_print_motor_structure( FILE *file, MX_RECORD *record )
{
	const char fname[] = "mxd_delta_motor_print_motor_structure()";

	MX_MOTOR *motor;
	MX_DELTA_MOTOR *delta_motor;
	double position, deadband;
	mx_status_type status;

	if ( record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
			"MX_RECORD pointer passed is NULL." );
	}

	motor = (MX_MOTOR *) (record->record_class_struct);

	if ( motor == (MX_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_MOTOR pointer for record '%s' is NULL.", record->name );
	}

	delta_motor = (MX_DELTA_MOTOR *) (record->record_type_struct);

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"MX_DELTA_MOTOR pointer for record '%s' is NULL.",
			record->name );
	}

	fprintf(file, "MOTOR parameters for motor '%s':\n", record->name);
	fprintf(file, "  Motor type      = DELTA_MOTOR.\n\n");

	fprintf(file, "  name            = %s\n", record->name);
	fprintf(file, "  moving motor    = %s\n",
				delta_motor->moving_motor_record->name);
	fprintf(file, "  fixed position  = %s\n",
				delta_motor->fixed_position_record->name);

	status = mx_motor_get_position( record, &position );

	if ( status.code != MXE_SUCCESS ) {
		return mx_error( MXE_FUNCTION_FAILED, fname,
			"Unable to read position of motor '%s'",
			record->name );
	}

	fprintf(file, "  position        = %g %s\n",
		position, motor->units);
	fprintf(file, "  delta scale    = %g %s per unscaled delta unit.\n",
		motor->scale, motor->units);
	fprintf(file, "  delta offset   = %g %s.\n",
		motor->offset, motor->units);
	fprintf(file, "  backlash        = %g %s.\n",
		motor->raw_backlash_correction.analog, motor->units);
	fprintf(file, "  negative limit  = %g %s.\n",
		motor->raw_negative_limit.analog, motor->units);
	fprintf(file, "  positive limit  = %g %s.\n",
		motor->raw_positive_limit.analog, motor->units);

	deadband = motor->scale * motor->deadband.analog;

	fprintf(file, "  deadband        = %g %s\n\n",
		deadband, motor->units );

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_delta_motor_read_parms_from_hardware( MX_RECORD *record )
{
	/* All saving of parameters is handled by the MX_MOTOR records
	 * this motor is derived from, so we need not do anything here.
	 */

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_delta_motor_write_parms_to_hardware( MX_RECORD *record )
{
	/* All setting of parameters is handled by the MX_MOTOR records
	 * this motor is derived from, so we need not do anything here.
	 */

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_delta_motor_open( MX_RECORD *record )
{
	/* Nothing to do. */

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_delta_motor_close( MX_RECORD *record )
{
	/* Nothing to do. */

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_delta_motor_motor_is_busy( MX_MOTOR *motor )
{
	const char fname[] = "mxd_delta_motor_motor_is_busy()";

	MX_DELTA_MOTOR *delta_motor;
	MX_RECORD *moving_motor_record;
	int busy;
	mx_status_type status;

	if ( motor == (MX_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_MOTOR pointer passed NULL.");
	}

	delta_motor = (MX_DELTA_MOTOR *) (motor->record->record_type_struct);

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"MX_DELTA_MOTOR pointer for record '%s' is NULL.",
			motor->record->name );
	}

	moving_motor_record = delta_motor->moving_motor_record;

	if ( moving_motor_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"Dependent MX_RECORD pointer for record '%s' is NULL.",
			motor->record->name );
	}

	status = mx_motor_is_busy( moving_motor_record, &busy );

	motor->busy = busy;

	return status;
}

MX_EXPORT mx_status_type
mxd_delta_motor_move_absolute( MX_MOTOR *motor )
{
	const char fname[] = "mxd_delta_motor_move_absolute()";

	MX_DELTA_MOTOR *delta_motor;
	MX_RECORD *moving_motor_record;
	double new_delta_position;
	double fixed_position_value;
	double new_moving_motor_position;
	mx_status_type status;

	if ( motor == (MX_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_MOTOR pointer passed NULL.");
	}

	delta_motor = (MX_DELTA_MOTOR *) (motor->record->record_type_struct);

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"MX_DELTA_MOTOR pointer for record '%s' is NULL.",
			motor->record->name );
	}

	moving_motor_record = delta_motor->moving_motor_record;

	if ( moving_motor_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"Moving motor record pointer for record '%s' is NULL.",
			motor->record->name );
	}

	/* Check the limits on this request. */

	new_delta_position = motor->raw_destination.analog;

	if ( new_delta_position > motor->raw_positive_limit.analog
	  || new_delta_position < motor->raw_negative_limit.analog )
	{
		return mx_error( MXE_WOULD_EXCEED_LIMIT, fname,
	"Requested position %g is outside the allowed range of %g to %g",
			new_delta_position, motor->raw_negative_limit.analog,
			motor->raw_positive_limit.analog );
	}

	/* Get the fixed position value. */

	status = mxd_delta_motor_get_fixed_position_value(
				delta_motor, &fixed_position_value );

	if ( status.code != MXE_SUCCESS )
		return status;

	/* Compute the new moving motor position. */

	new_moving_motor_position = fixed_position_value + new_delta_position;

	/* Now perform the move. */

	status = mx_motor_move_absolute( moving_motor_record,
				new_moving_motor_position, MXF_MTR_NOWAIT );

	return status;
}

MX_EXPORT mx_status_type
mxd_delta_motor_get_position( MX_MOTOR *motor )
{
	const char fname[] = "mxd_delta_motor_get_position()";

	MX_DELTA_MOTOR *delta_motor;
	MX_RECORD *moving_motor_record;
	double fixed_position_value;
	double moving_motor_position;
	mx_status_type status;

	if ( motor == (MX_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_MOTOR pointer passed NULL.");
	}

	delta_motor = (MX_DELTA_MOTOR *) (motor->record->record_type_struct);

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"MX_DELTA_MOTOR pointer for record '%s' is NULL.",
			motor->record->name );
	}

	moving_motor_record = delta_motor->moving_motor_record;

	if ( moving_motor_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"moving_motor_record pointer for record '%s' is NULL.",
			motor->record->name );
	}

	/* Get the current moving motor position. */

	status = mx_motor_get_position(
			moving_motor_record, &moving_motor_position );

	if ( status.code != MXE_SUCCESS )
		return status;

	/* Get the fixed position value. */

	status = mxd_delta_motor_get_fixed_position_value(
				delta_motor, &fixed_position_value );

	if ( status.code != MXE_SUCCESS )
		return status;

	/* Compute the pseudo motor position. */

	motor->raw_position.analog
				= moving_motor_position - fixed_position_value;

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_delta_motor_set_position( MX_MOTOR *motor )
{
	const char fname[] = "mxd_delta_motor_set_position()";

	return mx_error( MXE_UNSUPPORTED, fname,
	"'set position' is not valid for a delta motor." );
}

MX_EXPORT mx_status_type
mxd_delta_motor_soft_abort( MX_MOTOR *motor )
{
	const char fname[] = "mxd_delta_motor_soft_abort()";

	MX_DELTA_MOTOR *delta_motor;
	MX_RECORD *moving_motor_record;
	mx_status_type status;

	if ( motor == (MX_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_MOTOR pointer passed NULL.");
	}

	delta_motor = (MX_DELTA_MOTOR *) (motor->record->record_type_struct);

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"MX_DELTA_MOTOR pointer for record '%s' is NULL.",
			motor->record->name );
	}

	moving_motor_record = delta_motor->moving_motor_record;

	if ( moving_motor_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"moving_motor_record pointer for record '%s' is NULL.",
			motor->record->name );
	}

	status = mx_motor_soft_abort( moving_motor_record );

	return status;
}

MX_EXPORT mx_status_type
mxd_delta_motor_immediate_abort( MX_MOTOR *motor )
{
	const char fname[] = "mxd_delta_motor_immediate_abort()";

	MX_DELTA_MOTOR *delta_motor;
	MX_RECORD *moving_motor_record;
	mx_status_type status;

	if ( motor == (MX_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_MOTOR pointer passed NULL.");
	}

	delta_motor = (MX_DELTA_MOTOR *) (motor->record->record_type_struct);

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"MX_DELTA_MOTOR pointer for record '%s' is NULL.",
			motor->record->name );
	}

	moving_motor_record = delta_motor->moving_motor_record;

	if ( moving_motor_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"moving_motor_record pointer for record '%s' is NULL.",
			motor->record->name );
	}

	status = mx_motor_immediate_abort( moving_motor_record );

	return status;
}

MX_EXPORT mx_status_type
mxd_delta_motor_positive_limit_hit( MX_MOTOR *motor )
{
	const char fname[] = "mxd_delta_motor_positive_limit_hit()";

	MX_DELTA_MOTOR *delta_motor;
	MX_RECORD *moving_motor_record;
	int limit_hit;
	mx_status_type status;

	if ( motor == (MX_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_MOTOR pointer passed NULL.");
	}

	delta_motor = (MX_DELTA_MOTOR *) (motor->record->record_type_struct);

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"MX_DELTA_MOTOR pointer for record '%s' is NULL.",
			motor->record->name );
	}

	moving_motor_record = delta_motor->moving_motor_record;

	if ( moving_motor_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"moving_motor_record pointer for record '%s' is NULL.",
			motor->record->name );
	}

	if ( motor->scale >= 0.0 ) {
		status = mx_motor_positive_limit_hit(
				moving_motor_record, &limit_hit);
	} else {
		status = mx_motor_negative_limit_hit(
				moving_motor_record, &limit_hit);
	}

	motor->positive_limit_hit = limit_hit;

	return status;
}

MX_EXPORT mx_status_type
mxd_delta_motor_negative_limit_hit( MX_MOTOR *motor )
{
	const char fname[] = "mxd_delta_motor_negative_limit_hit()";

	MX_DELTA_MOTOR *delta_motor;
	MX_RECORD *moving_motor_record;
	int limit_hit;
	mx_status_type status;

	if ( motor == (MX_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_MOTOR pointer passed NULL.");
	}

	delta_motor = (MX_DELTA_MOTOR *) (motor->record->record_type_struct);

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"MX_DELTA_MOTOR pointer for record '%s' is NULL.",
			motor->record->name );
	}

	moving_motor_record = delta_motor->moving_motor_record;

	if ( moving_motor_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"moving_motor_record pointer for record '%s' is NULL.",
			motor->record->name );
	}

	if ( motor->scale >= 0.0 ) {
		status = mx_motor_negative_limit_hit(
				moving_motor_record, &limit_hit);
	} else {
		status = mx_motor_positive_limit_hit(
				moving_motor_record, &limit_hit);
	}

	motor->negative_limit_hit = limit_hit;

	return status;
}

MX_EXPORT mx_status_type
mxd_delta_motor_find_home_position( MX_MOTOR *motor )
{
	const char fname[] = "mxd_delta_motor_find_home_position()";

	return mx_error( MXE_UNSUPPORTED, fname,
	"Finding home for a delta motor is _not_ allowed.  Motor name = '%s'",
		motor->record->name );
}

/*======== Private functions for the use of the driver only. ========*/

static mx_status_type
mxd_delta_motor_get_fixed_position_value(
		MX_DELTA_MOTOR *delta_motor,
		double *fixed_position_value )
{
	const char fname[] = "mxd_delta_motor_get_fixed_position()";

	MX_RECORD *fixed_position_record;
	void *pointer_to_value;
	mx_status_type status;

	if ( delta_motor == (MX_DELTA_MOTOR *) NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
			"NULL pointer passed." );
	}

	fixed_position_record = delta_motor->fixed_position_record;

	if ( fixed_position_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
	"fixed_position_record pointer for 'delta_motor' record '%s' is NULL.",
			delta_motor->motor->record->name );
	}

	/* Get the fixed position value. */

	switch ( fixed_position_record->mx_superclass ) {
	case MXR_VARIABLE:
		switch( fixed_position_record->mx_type ) {
		case MXV_NET_DOUBLE:
		case MXV_EPI_DOUBLE:
			status = mx_receive_variable( fixed_position_record );

			if ( status.code != MXE_SUCCESS )
				return status;

			/* Fall through to the next case. */

		case MXV_INL_DOUBLE:
			status = mx_get_variable_pointer(
				fixed_position_record, &pointer_to_value );

			if ( status.code != MXE_SUCCESS )
				return status;

			*fixed_position_value = *((double *) pointer_to_value);

			break;
		default:
			return mx_error( MXE_UNSUPPORTED, fname,
		"The variable type %ld for record '%s' is not supported "
		"for the fixed position of a 'delta_motor' in record '%s'.",
				fixed_position_record->mx_type,
				fixed_position_record->name,
				delta_motor->motor->record->name );
			break;
		}
		break;
	case MXR_DEVICE:
		switch( fixed_position_record->mx_class ) {
		case MXC_MOTOR:
			status = mx_motor_get_position(
				fixed_position_record, fixed_position_value );

			if ( status.code != MXE_SUCCESS )
				return status;

			break;
		default:
			return mx_error( MXE_UNSUPPORTED, fname,
		"The device class %ld for record '%s' is not supported "
		"for the fixed position of a 'delta_motor' in record '%s'.",
				fixed_position_record->mx_class,
				fixed_position_record->name,
				delta_motor->motor->record->name );
			break;
		}
		break;
	default:
		return mx_error( MXE_UNSUPPORTED, fname,
		"The record superclass %ld for record '%s' is not supported "
		"for the fixed position of a 'delta_motor' in record '%s'.",
			fixed_position_record->mx_superclass,
			fixed_position_record->name,
			delta_motor->motor->record->name );
		break;
	}

	return MX_SUCCESSFUL_RESULT;
}

