/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mx_amp.h
 *
 * Purpose: MX function library of generic amplifier operations.
 */

#include <stdio.h>

#include "mxconfig.h"
#include "mx_rec.h"
#include "mx_drivr.h"
#include "mx_amp.h"

/*=======================================================================*/

/* This function is a utility function to consolidate all of the pointer
 * mangling that often has to happen at the beginning of an mx_amplifier_...
 * function.  The parameter 'calling_fname' is passed so that error messages
 * will appear with the name of the calling function.
 */

static mx_status_type
mx_amplifier_get_pointers( MX_RECORD *amplifier_record,
			MX_AMPLIFIER **amplifier,
			MX_AMPLIFIER_FUNCTION_LIST **function_list_ptr,
			const char *calling_fname )
{
	const char fname[] = "mx_amplifier_get_pointers()";

	if ( amplifier_record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
			"The amplifier_record pointer passed by '%s' was NULL",
			calling_fname );
	}

	if ( amplifier_record->mx_class != MXC_AMPLIFIER ) {
		return mx_error( MXE_TYPE_MISMATCH, fname,
			"The record '%s' passed by '%s' is not a amplifier.",
			amplifier_record->name, calling_fname );
	}

	if ( amplifier != (MX_AMPLIFIER **) NULL ) {
		*amplifier = (MX_AMPLIFIER *)
				(amplifier_record->record_class_struct);

		if ( *amplifier == (MX_AMPLIFIER *) NULL ) {
			return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
	"The MX_AMPLIFIER pointer for record '%s' passed by '%s' is NULL",
				amplifier_record->name, calling_fname );
		}
	}

	if ( function_list_ptr != (MX_AMPLIFIER_FUNCTION_LIST **) NULL ) {
		*function_list_ptr = (MX_AMPLIFIER_FUNCTION_LIST *)
			(amplifier_record->class_specific_function_list);

		if (*function_list_ptr == (MX_AMPLIFIER_FUNCTION_LIST *) NULL){
			return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
"MX_AMPLIFIER_FUNCTION_LIST pointer for record '%s' passed by '%s' is NULL.",
				amplifier_record->name, calling_fname );
		}
	}

	return MX_SUCCESSFUL_RESULT;
}

/*=======================================================================*/

MX_EXPORT mx_status_type
mx_amplifier_get_gain( MX_RECORD *amplifier_record, double *gain )
{
	const char fname[] = "mx_amplifier_get_gain()";

	MX_AMPLIFIER *amplifier;
	MX_AMPLIFIER_FUNCTION_LIST *function_list;
	mx_status_type ( *get_gain_fn ) ( MX_AMPLIFIER * );
	mx_status_type status;

	status = mx_amplifier_get_pointers( amplifier_record,
					&amplifier, &function_list, fname );

	if ( status.code != MXE_SUCCESS )
		return status;

	get_gain_fn = function_list->get_gain;

	if ( get_gain_fn == NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"get_gain function ptr for amplifier record '%s' is NULL.",
			amplifier_record);
	}

	status = (*get_gain_fn)( amplifier );

	*gain = amplifier->gain;

	return status;
}

MX_EXPORT mx_status_type
mx_amplifier_set_gain( MX_RECORD *amplifier_record, double gain )
{
	const char fname[] = "mx_amplifier_set_gain()";

	MX_AMPLIFIER *amplifier;
	MX_AMPLIFIER_FUNCTION_LIST *function_list;
	mx_status_type ( *set_gain_fn ) ( MX_AMPLIFIER * );
	mx_status_type status;

	status = mx_amplifier_get_pointers( amplifier_record,
					&amplifier, &function_list, fname );

	if ( status.code != MXE_SUCCESS )
		return status;

	set_gain_fn = function_list->set_gain;

	if ( set_gain_fn == NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"set_gain function ptr for amplifier record '%s' is NULL.",
			amplifier_record);
	}

	amplifier->gain = gain;

	status = (*set_gain_fn)( amplifier );

	return status;
}

MX_EXPORT mx_status_type
mx_amplifier_get_offset( MX_RECORD *amplifier_record, double *offset )
{
	const char fname[] = "mx_amplifier_get_offset()";

	MX_AMPLIFIER *amplifier;
	MX_AMPLIFIER_FUNCTION_LIST *function_list;
	mx_status_type ( *get_offset_fn ) ( MX_AMPLIFIER * );
	mx_status_type status;

	status = mx_amplifier_get_pointers( amplifier_record,
					&amplifier, &function_list, fname );

	if ( status.code != MXE_SUCCESS )
		return status;

	get_offset_fn = function_list->get_offset;

	if ( get_offset_fn == NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"get_offset function ptr for amplifier record '%s' is NULL.",
			amplifier_record);
	}

	status = (*get_offset_fn)( amplifier );

	*offset = amplifier->offset;

	return status;
}

MX_EXPORT mx_status_type
mx_amplifier_set_offset( MX_RECORD *amplifier_record, double offset )
{
	const char fname[] = "mx_amplifier_set_offset()";

	MX_AMPLIFIER *amplifier;
	MX_AMPLIFIER_FUNCTION_LIST *function_list;
	mx_status_type ( *set_offset_fn ) ( MX_AMPLIFIER * );
	mx_status_type status;

	status = mx_amplifier_get_pointers( amplifier_record,
					&amplifier, &function_list, fname );

	if ( status.code != MXE_SUCCESS )
		return status;

	set_offset_fn = function_list->set_offset;

	if ( set_offset_fn == NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"set_offset function ptr for amplifier record '%s' is NULL.",
			amplifier_record);
	}

	amplifier->offset = offset;

	status = (*set_offset_fn)( amplifier );

	return status;
}

MX_EXPORT mx_status_type
mx_amplifier_get_time_constant( MX_RECORD *amplifier_record,
						double *time_constant )
{
	const char fname[] = "mx_amplifier_get_time_constant()";

	MX_AMPLIFIER *amplifier;
	MX_AMPLIFIER_FUNCTION_LIST *function_list;
	mx_status_type ( *get_time_constant_fn ) ( MX_AMPLIFIER * );
	mx_status_type status;

	status = mx_amplifier_get_pointers( amplifier_record,
					&amplifier, &function_list, fname );

	if ( status.code != MXE_SUCCESS )
		return status;

	get_time_constant_fn = function_list->get_time_constant;

	if ( get_time_constant_fn == NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
	"get_time_constant function ptr for amplifier record '%s' is NULL.",
			amplifier_record);
	}

	status = (*get_time_constant_fn)( amplifier );

	*time_constant = amplifier->time_constant;

	return status;
}

MX_EXPORT mx_status_type
mx_amplifier_set_time_constant( MX_RECORD *amplifier_record,
						double time_constant )
{
	const char fname[] = "mx_amplifier_set_time_constant()";

	MX_AMPLIFIER *amplifier;
	MX_AMPLIFIER_FUNCTION_LIST *function_list;
	mx_status_type ( *set_time_constant_fn ) ( MX_AMPLIFIER * );
	mx_status_type status;

	status = mx_amplifier_get_pointers( amplifier_record,
					&amplifier, &function_list, fname );

	if ( status.code != MXE_SUCCESS )
		return status;

	set_time_constant_fn = function_list->set_time_constant;

	if ( set_time_constant_fn == NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
	"set_time_constant function ptr for amplifier record '%s' is NULL.",
			amplifier_record);
	}

	amplifier->time_constant = time_constant;

	status = (*set_time_constant_fn)( amplifier );

	return status;
}

