/*
 * Name:    mx_error.c
 *
 * Purpose: Functions for displaying error output.
 *
 *----------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "mx_util.h"

typedef struct {
	int error_code;
	char error_message_text[30];
} MX_ERROR_MESSAGE;

static MX_ERROR_MESSAGE error_message_list[] = {
{ MXE_SUCCESS,			"MXE_SUCCESS" },
{ MXE_NULL_ARGUMENT,		"MXE_NULL_ARGUMENT" },
{ MXE_ILLEGAL_ARGUMENT,		"MXE_ILLEGAL_ARGUMENT" },
{ MXE_CORRUPT_DATA_STRUCTURE,	"MXE_CORRUPT_DATA_STRUCTURE" },
{ MXE_UNPARSEABLE_STRING,	"MXE_UNPARSEABLE_STRING" },
{ MXE_END_OF_DATA,		"MXE_END_OF_DATA" },
{ MXE_UNEXPECTED_END_OF_DATA,	"MXE_UNEXPECTED_END_OF_DATA" },
{ MXE_UNDEFINED_RECORD,		"MXE_UNDEFINED_RECORD" },
{ MXE_TYPE_MISMATCH,		"MXE_TYPE_MISMATCH" },
{ MXE_NOT_YET_IMPLEMENTED,	"MXE_NOT_YET_IMPLEMENTED" },
{ MXE_UNSUPPORTED,		"MXE_UNSUPPORTED" },
{ MXE_OUT_OF_MEMORY,		"MXE_OUT_OF_MEMORY" },
{ MXE_WOULD_EXCEED_LIMIT,	"MXE_WOULD_EXCEED_LIMIT" },
{ MXE_LIMIT_WAS_EXCEEDED,	"MXE_LIMIT_WAS_EXCEEDED" },
{ MXE_INTERFACE_IO_ERROR,	"MXE_INTERFACE_IO_ERROR" },
{ MXE_DEVICE_IO_ERROR,		"MXE_DEVICE_IO_ERROR" },
{ MXE_FILE_IO_ERROR,		"MXE_FILE_IO_ERROR" },
{ MXE_TERMINAL_IO_ERROR,	"MXE_TERMINAL_IO_ERROR" },
{ MXE_IPC_IO_ERROR,		"MXE_IPC_IO_ERROR" },
{ MXE_IPC_CONNECTION_LOST,	"MXE_IPC_CONNECTION_LOST" },
{ MXE_NETWORK_IO_ERROR,		"MXE_NETWORK_IO_ERROR" },
{ MXE_NETWORK_CONNECTION_LOST,	"MXE_NETWORK_CONNECTION_LOST" },
{ MXE_NOT_READY,		"MXE_NOT_READY" },
{ MXE_INTERRUPTED,		"MXE_INTERRUPTED" },
{ MXE_PAUSE_REQUESTED,		"MXE_PAUSE_REQUESTED" },
{ MXE_INTERFACE_ACTION_FAILED,	"MXE_INTERFACE_ACTION_FAILED" },
{ MXE_DEVICE_ACTION_FAILED,	"MXE_DEVICE_ACTION_FAILED" },
{ MXE_FUNCTION_FAILED,		"MXE_FUNCTION_FAILED" },
{ MXE_CONTROLLER_INTERNAL_ERROR,"MXE_CONTROLLER_INTERNAL_ERROR" },
{ MXE_PERMISSION_DENIED,        "MXE_PERMISSION_DENIED" },
{ MXE_CLIENT_REQUEST_DENIED,	"MXE_CLIENT_REQUEST_DENIED" },
{ MXE_TRY_AGAIN,		"MXE_TRY_AGAIN" },
{ 0, "" } };

static void (*mx_error_output_function)( char * )
					= mx_error_default_output_function;

#if ( ! USE_STACK_BASED_MX_ERROR )

static char mx_error_message_buffer[MXU_ERROR_MESSAGE_LENGTH + 1];

#endif /* USE_STACK_BASED_MX_ERROR */

MX_EXPORT mx_status_type
mx_error( long error_code, const char *location, char *format, ... )
{
	mx_status_type status_struct;

	va_list args;
	char buffer1[500];
	char buffer2[500];
	long i;
	static long num_messages
		= sizeof(error_message_list) / sizeof(error_message_list[0]);

	va_start(args, format);
	vsprintf(buffer1, format, args);
	va_end(args);

	/* Fill in the status structure. */

	status_struct.code = error_code;
	status_struct.location = location;

#if USE_STACK_BASED_MX_ERROR

	strncpy( status_struct.message, buffer1, MXU_ERROR_MESSAGE_LENGTH );

#else /* not USE_STACK_BASED_MX_ERROR */

	strncpy( mx_error_message_buffer, buffer1, MXU_ERROR_MESSAGE_LENGTH );

	status_struct.message = &mx_error_message_buffer[0];

#endif /* USE_STACK_BASED_MX_ERROR */

	/* Format the optional error message. */

	if ( mx_error_output_function != NULL ) {

		i = error_code - 1000;

		if ( i >= 0 && i < num_messages ) {
			sprintf( buffer2,
				"%s in %s:\n-> %s",
				error_message_list[i].error_message_text,
				location, buffer1 );
		} else {
			sprintf( buffer2,
		"Unrecognized error code %ld occurred in '%s':\n-> %s",
				error_code, location, buffer1 );
		}

		(*mx_error_output_function)( buffer2 );
	}
	return status_struct;
}

/* mx_error_quiet() is just like mx_error() except that it doesn't
 * invoke *mx_error_output_function().
 */

MX_EXPORT mx_status_type
mx_error_quiet( long error_code, const char *location, char *format, ... )
{
	mx_status_type status_struct;

	va_list args;
	char buffer1[500];

	va_start(args, format);
	vsprintf(buffer1, format, args);
	va_end(args);

	/* Fill in the status structure. */

	status_struct.code = error_code;
	status_struct.location = location;

#if USE_STACK_BASED_MX_ERROR

	strncpy( status_struct.message, buffer1, MXU_ERROR_MESSAGE_LENGTH );

#else /* not USE_STACK_BASED_MX_ERROR */

	strncpy( mx_error_message_buffer, buffer1, MXU_ERROR_MESSAGE_LENGTH );

	status_struct.message = &mx_error_message_buffer[0];

#endif /* USE_STACK_BASED_MX_ERROR */

	return status_struct;
}

MX_EXPORT char *
mx_strerror( long error_code, char *buffer, size_t max_buffer_length )
{
	static char local_buffer[80];
	char *ptr;
	long i;
	static long num_messages
		= sizeof(error_message_list) / sizeof(error_message_list[0]);

	if ( buffer != NULL ) {
		ptr = buffer;
	} else {
		ptr = local_buffer;
		max_buffer_length = sizeof( local_buffer );
	}

	i = error_code - 1000;

	if ( i >= 0 && i < num_messages ) {
		mx_strncpy( ptr, error_message_list[i].error_message_text,
					max_buffer_length );
	} else {
		sprintf( ptr, "MXE_(%ld)", error_code );
	}
	return ptr;
}

MX_EXPORT mx_status_type
mx_successful_result( void )
{
	mx_status_type status;

	status.code = MXE_SUCCESS;
	status.location = NULL;

#if USE_STACK_BASED_MX_ERROR

	strcpy( status.message, "" );

#else /* not USE_STACK_BASED_MX_ERROR */

	status.message = NULL;

#endif /* USE_STACK_BASED_MX_ERROR */

	return status;
}
	
MX_EXPORT void
mx_set_error_output_function( void (*output_function)(char *) )
{
	if ( output_function == NULL ) {
		mx_error_output_function = mx_error_default_output_function;
	} else {
		mx_error_output_function = output_function;
	}
	return;
}

MX_EXPORT void
mx_error_default_output_function( char *string )
{
	fprintf( stderr, "%s\n", string );

	return;
}

