/*
 * Name:    mx_util.h
 *
 * Purpose: Define utility functions and generally used symbols.
 *
 *---------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __MX_UTIL_H__
#define __MX_UTIL_H__

#include <string.h>	/* We get size_t from here. */

/*
 * Macros for declaring shared library or DLL functions.
 *
 * The MX_API and MX_EXPORT macros are used for the declaration of functions
 * in shared libraries or DLLs.  They must be used by publically visible
 * functions in a shared library or DLL and must _not_ be used by functions
 * that are not in a shared library or DLL.
 *
 * MX_API and MX_EXPORT are similar but not the same.  MX_API is used only
 * in header files (.h files), while MX_EXPORT is used only in .c files
 * where the body of the function appears.  On Win32, these are identical,
 * but this will not be true, in general, on any other platform.
 */

#if defined(OS_WIN32)
#   ifdef __MX_LIBRARY__
#	define MX_API		_declspec(dllexport)
#	define MX_EXPORT	_declspec(dllexport)
#   else
#	define MX_API		_declspec(dllimport)
#	define MX_EXPORT	__ERROR_ONLY_USE_THIS_IN_LIBRARIES__
#   endif
#elif defined(OS_VMS)
#       define MX_API		extern
#       define MX_EXPORT
#else
#   ifdef __MX_LIBRARY__
#	define MX_API		extern
#	define MX_EXPORT
#   else
#	define MX_API		extern
#	define MX_EXPORT	__ERROR_ONLY_USE_THIS_IN_LIBRARIES__
#   endif
#endif

#ifndef SUCCESS
#define INTERRUPTED 2
#define SUCCESS     1
#define FAILURE     0
#endif

#ifndef TRUE
#define TRUE	1
#define FALSE	0
#endif

#define MXU_STRING_LENGTH       20
#define MXU_FILENAME_LENGTH	200
#define MXU_BUFFER_LENGTH	400
#define MXU_HOSTNAME_LENGTH	100

MX_API void mx_sleep( unsigned long seconds );
MX_API void mx_msleep( unsigned long milliseconds );
MX_API void mx_usleep( unsigned long microseconds );

/* mx_round() rounds to the nearest integer. */

MX_API long mx_round( double value );

/* mx_divide_safely divides two floating point numbers by each other
 * while testing for and avoiding division by zero.
 */

MX_API double mx_divide_safely( double numerator, double denominator );

/* mx_difference() computes a relative difference function. */

MX_API double mx_difference( double value1, double value2 );

/* mx_match() does simple wildcard matching. */

MX_API int mx_match( char *pattern, char *string );

/* mx_strncpy() is a replacement for strncpy() that ensures that the
 * string is '\0' terminated.
 */

#define mx_strncpy( dest, src, n ) \
			do { \
				*(dest) = '\0'; \
				strncat( (dest), (src), (n) ); \
			} while (0)

/* mx_free() calls free() and then makes sure that the pointer we just
 * free'd has been set to NULL.
 */

#define mx_free( ptr ) \
			do { \
				free( ptr ); \
				(ptr) = NULL; \
			} while (0)

/* == Debugging functions. == */

/* Note that in any call to MX_DEBUG(), _all_ the arguments together 
 * after the debug level are enclosed in _one_ extra set of parentheses.
 * If you don't do this, the code will not compile.
 *
 * For example,
 *    MX_DEBUG( 2, ("The current value of foo is %d\n", foo) );
 * 
 * This is so that the preprocessor will treat everything after "2,"
 * as one _big_ macro argument.  This is the closest one can come to
 * "varargs" macros in standard ANSI C.  Also, the line below that
 * reads "mx_debug_function  text ;" is _not_ in error and is _not_
 * missing any needed parentheses.  The preprocessor will get the
 * needed parentheses from the extra set of parentheses in the
 * original invocation of MX_DEBUG().  Go read up on just how macro
 * expansion by the C preprocessor is supposed to work if you want
 * to understand this better.  Also, the Frequently Asked Questions
 * file for comp.lang.c on Usenet has a few words about this.
 *
 * Yes, it's a trick, but it's a trick that works very well as long
 * you don't forget those extra set of parentheses.  Incidentally,
 * you still need the parentheses even if the format field in the
 * second argument is a constant string with no %'s in it.  That's
 * because if you don't you'll end up after the macro expansion
 * with something like:   mx_debug_function "This doesn't work" ;
 */

MX_API void mx_debug_function( char *format, ... );

#ifndef DEBUG
#define MX_DEBUG( level, text )
#else
#define MX_DEBUG( level, text ) \
		if ( (level) <= mx_get_debug_level() )  { \
			mx_debug_function  text ; \
		}
#endif

MX_API void mx_set_debug_level( int debug_level );
MX_API int  mx_get_debug_level( void );

MX_API void mx_set_debug_output_function( void (*)( char * ) );
MX_API void mx_debug_default_output_function( char *string );

/* === User interrupts. === */

#define MXF_USER_INT_NONE	0
#define MXF_USER_INT_ABORT	1
#define MXF_USER_INT_PAUSE	2

#define MXF_USER_INT_ERROR	(-1)

MX_API int  mx_user_requested_interrupt( void );
MX_API void mx_set_user_interrupt_function( int (*)( void ) );
MX_API int  mx_default_user_interrupt_function( void );

/* === Status reporting. === */

MX_API void mx_info( char *format, ... );
MX_API void mx_info_dialog( char *text_prompt, char *gui_prompt,
					char *button_label );

MX_API void mx_set_info_output_function( void (*)( char * ) );
MX_API void mx_info_default_output_function( char *string );

MX_API void mx_set_info_dialog_function( void (*)( char *, char *, char * ) );
MX_API void mx_info_default_dialog_function( char *text_prompt,
					char *gui_prompt, char *button_label );

MX_API void mx_startup_banner( char *buffer );

/* === Error reporting. === */

#define MXU_ERROR_MESSAGE_LENGTH	200

#if USE_STACK_BASED_MX_ERROR

typedef struct {
	long code;		/* The error code. */
	const char *location;	/* Function name where the error occurred. */
	char message[MXU_ERROR_MESSAGE_LENGTH+1]; /* The specific error msg.*/
} mx_status_type;

#else /* not USE_STACK_BASED_MX_ERROR */

typedef struct {
	long code;		/* The error code. */
	const char *location;	/* Function name where the error occurred. */
	char *message;                            /* The specific error msg.*/
} mx_status_type;

#endif /* USE_STACK_BASED_MX_ERROR */

MX_API mx_status_type mx_error( long error_code,
		const char *location, char *format, ... );

MX_API mx_status_type mx_error_quiet( long error_code,
		const char *location, char *format, ... );

MX_API char *mx_strerror(long error_code, char *buffer, size_t buffer_length);

MX_API void mx_set_error_output_function( void (*)( char * ) );
MX_API void mx_error_default_output_function( char *string );

MX_API mx_status_type mx_successful_result( void );

#define MX_SUCCESSFUL_RESULT	mx_successful_result()

MX_API char *mx_username( char *buffer, size_t buffer_length );

MX_API int mx_get_max_file_descriptors( void );

MX_API char *mx_current_time_string( void );

MX_API char *mx_skip_string_fields( char *buffer, int num_fields );

/* === Define error message numbers. === */

#define MXE_SUCCESS			1000		/* No error. */

#define MXE_NULL_ARGUMENT		1001
#define MXE_ILLEGAL_ARGUMENT		1002
#define MXE_CORRUPT_DATA_STRUCTURE	1003
#define MXE_UNPARSEABLE_STRING		1004
#define MXE_END_OF_DATA			1005
#define MXE_UNEXPECTED_END_OF_DATA	1006
#define MXE_UNDEFINED_RECORD		1007
#define MXE_TYPE_MISMATCH		1008
#define MXE_NOT_YET_IMPLEMENTED		1009
#define MXE_UNSUPPORTED			1010
#define MXE_OUT_OF_MEMORY		1011
#define MXE_WOULD_EXCEED_LIMIT		1012
#define MXE_LIMIT_WAS_EXCEEDED		1013
#define MXE_INTERFACE_IO_ERROR		1014
#define MXE_DEVICE_IO_ERROR		1015
#define MXE_FILE_IO_ERROR		1016
#define MXE_TERMINAL_IO_ERROR		1017
#define MXE_IPC_IO_ERROR		1018
#define MXE_IPC_CONNECTION_LOST		1019
#define MXE_NETWORK_IO_ERROR		1020
#define MXE_NETWORK_CONNECTION_LOST	1021
#define MXE_NOT_READY			1022
#define MXE_INTERRUPTED			1023
#define MXE_PAUSE_REQUESTED		1024
#define MXE_INTERFACE_ACTION_FAILED	1025
#define MXE_DEVICE_ACTION_FAILED	1026
#define MXE_FUNCTION_FAILED		1027
#define MXE_CONTROLLER_INTERNAL_ERROR	1028
#define MXE_PERMISSION_DENIED		1029
#define MXE_CLIENT_REQUEST_DENIED	1030
#define MXE_TRY_AGAIN			1031

#endif /* __MX_UTIL_H__ */
